#include "dynarr.h"
#include <string.h>

int
dynarr_init(dynarr_t *d, size_t n)
{
    void *data = calloc(n, 1);
    if(data == NULL) return -1;

    d->data = data;
    d->cap = n;
    d->sz = 0;

    return 0;
}

int
dynarr_release(dynarr_t *d)
{
    free(d->data);
    return 0;
}

int
dynarr_reserve(dynarr_t *d, size_t cap)
{
    if(d->cap >= cap) return 0;

    void *ndata = realloc(d->data, cap);
    if(ndata == NULL) return -1;

    d->cap = cap;
    d->data = ndata;
    return 0;
}

int
dynarr_clear(dynarr_t *d, u_int8_t c)
{
    memset(d->data, c, d->cap);
    d->sz = 0;
    return 0;
}

int
dynarr_shrink(dynarr_t *d)
{
    if(d->sz == d->cap) return 0;

    void *ndata = realloc(d->data, d->sz);
    if(ndata == NULL) return -1;

    d->cap = d->sz;
    d->data = ndata;
    return 0;
}

int
dynarr_append(dynarr_t *d, void *buf, size_t sz)
{
    if(d->sz + sz > d->cap)
	dynarr_reserve(d,(d->sz + sz) < (2 * d->sz)
		       ? (2 * d->sz) : (d->sz + sz));
    memcpy(d->data + d->sz, buf, sz);
    d->sz += sz;
    return 0;
}
