#ifndef RPC_DYNARR_H
#define RPC_DYNARR_H

#include <stdlib.h>

typedef struct dynarr_s {
    void *data;
    size_t sz, cap;
} dynarr_t;

/**
 * Initializes a dynarr_t with at least #n bytes of capacity.
 * Returns -1 if unable to allocate enough memory.
 */
int
dynarr_init(dynarr_t *d, size_t n);

/**
 * Releases allocated memory associated with dynarr_t #d.
 */
int
dynarr_release(dynarr_t *d);

/**
 * Fill the buffer #d with character #c and set its size to 0.
 */
int
dynarr_clear(dynarr_t *d, u_int8_t c);

/**
 * Reserves at least #cap bytes in #d's buffer.
 * Returns -1 if unable to allocate enough memory.
 */
int
dynarr_reserve(dynarr_t *d, size_t cap);

/**
 * Shrinks #d's buffer to be as small as the data inside. No-op if capacity
 * matches size.
 */
int
dynarr_shrink(dynarr_t *d);

/**
 * Copies over #sz bytes from #buf to the end of #d's data, possibly expanding
 * the buffer by at least the amount necessary to store both the data and #buf.
 *
 * Returns -1 if unable to allocate enough memory.
 */
int
dynarr_append(dynarr_t *d, void *buf, size_t sz);

#endif // RPC_DYNARR_H
