#include <stdio.h>
#include <uv.h>
#include <stdlib.h>
#include <signal.h>

#include "../server.h"
#include "../transport/tcp.h"

static uv_loop_t loop;
static uv_tcp_t server_sock;
static rpc_tcp_server_t *server;

static void
catch_sigint(int signal)
{
    rpc_tcp_close(server, NULL, NULL);
    uv_loop_close(&loop);
    exit(0);
}

static void
on_msg(rpc_tcp_stream_t *stream, int status, void *buf, ssize_t n)
{
    if(status < 0) {
	fprintf(stderr, "on_msg() recieved status = %d, %s\n",
		status, uv_err_name(status));
	if(n == -1)
	    rpc_tcp_conn_close((rpc_tcp_conn_t*)stream, NULL, NULL);
	return;
    }

    printf("on_msg() recieved message on stream (%d): ",
	   stream == NULL ? 0 : stream->id);
    printf("%.*s\n", (int)n, (char *)buf);

    /* rpc_tcp_conn_close(stream->owner, NULL, NULL); */
    free(buf);
}

static void
on_new_conn(rpc_tcp_conn_t *conn, int status)
{
    rpc_tcp_conn_read_start(conn, on_msg);
}

int
main()
{
    uv_loop_init(&loop);

    printf("rpc: main() started\n\n");

    uv_tcp_t server_sock;
    uv_tcp_init(&loop, &server_sock);

    struct sockaddr_in s;
    uv_ip4_addr("0.0.0.0", 4321, &s);
    uv_tcp_bind(&server_sock, (const struct sockaddr*)&s, 0);

    server = rpc_tcp_listen(&server_sock, 10, on_new_conn);
    signal(SIGINT, catch_sigint);

    uv_run(&loop, UV_RUN_DEFAULT);

    uv_loop_close(&loop);
    return 0;
}
